from pytypecho import Page


def test_get_page(te):
    r = te.get_page(2)
    assert r


def test_get_pages(te):
    r = te.get_pages()
    assert r


def test_new_page(te):
    page = Page(title='Page Title', description='Page Description')
    r = te.new_page(page, publish=True)
    assert r
    assert r.isdigit()


def test_edit_page(te):
    page = Page(title='Page Title', description='Page Description')
    num = te.new_page(page, publish=True)
    page_edited = Page(title='Edited Page Title', description='Edited Page Description')
    r = te.edit_page(page_edited, page_id=int(num), publish=True)
    assert r
    assert r == num


def test_del_post(te):
    page = Page(title='Del Page Title', description='Del Page Description')
    num = te.new_page(page, publish=True)
    r = te.del_page(int(num))
    assert r



from pytypecho import Category


def test_get_category(te):
    r = te.get_categories()
    assert r
    assert r[0]['categoryName'] == '默认分类'


def test_new_category(te):
    category = Category(name='Category Name')
    r = te.new_category(category)
    assert r
    assert r.isdigit()


def del_category(te):
    r = te.del_category(2)
    assert r



from pytypecho import Post


def test_get_post(te):
    r = te.get_post(1)
    assert r


def test_get_posts(te):
    r = te.get_posts()
    assert r


def test_new_post(te):
    post = Post(title='Post Title', description='Post Description')
    r = te.new_post(post, publish=True)
    assert r
    assert r.isdigit()


def test_edit_post(te):
    post = Post(title='Post Title', description='Post Description')
    num = te.new_post(post, publish=True)
    post_edited = Post(title='Edited Post Title', description='Edited Post Description')
    r = te.edit_post(post_edited, post_id=int(num), publish=True)
    assert r
    assert r == num


def test_del_post(te):
    post = Post(title='Del Post Title', description='Del Post Description')
    num = te.new_post(post, publish=True)
    r = te.del_post(int(num))
    assert r is None



from dataclasses import dataclass, field
from typing import List, BinaryIO
from datetime import datetime


@dataclass
class Meta:
    name: str
    parent: int = 0
    slug: str = ''
    description: str = ''


@dataclass
class Category(Meta):
    pass


@dataclass
class Tag(Meta):
    pass


@dataclass
class Content:
    """
    Post needs at least: title, description.
    Page needs at least: title, description.

    text_more will be connected with description as description+'\n<!--more-->\n'+text_more by Typecho.
    tags should be split by ',' like 'tag1, tag2'
    dateCreated can be datetime.datetime.
    allow_feed has no effect because Typecho not use
    status could be 'publish' or 'save' or 'private'.
    """
    title: str
    description: str

    slug: str = ''
    mt_text_more: str = ''
    wp_password: str = ''
    mt_keywords: str = ''
    dateCreated: datetime = datetime.now()
    mt_allow_comments: int = 1
    mt_allow_pings: int = 1
    post_status: str = ''


@dataclass
class Post(Content):
    post_type: str = 'post'
    categories: List[str] = field(default_factory=list)


@dataclass
class Page(Content):
    post_type: str = 'page'
    wp_page_order: int = 0
    wp_page_template: str = ''


@dataclass
class Attachment:
    name: str
    bytes: BinaryIO


@dataclass
class Comment:
    content: str

    author: str = ''
    author_email: str = ''
    author_url: str = ''

    comment_author: int = 0
    comment_author_email: int = 0
    comment_author_url: int = 0

    def __post_init__(self):
        if self.author:
            self.comment_author = 1
        if self.author_email:
            self.comment_author_email = 1
        if self.author_url:
            self.comment_author_url = 1



from logging import getLogger, StreamHandler, Formatter

fmt = Formatter('%(levelname)s: [%(asctime)s: %(message)s')
sh = StreamHandler()
sh.setFormatter(fmt)
logger = getLogger()
logger.addHandler(sh)



from pytypecho import Comment


def test_get_comment(te):
    r = te.get_comment(1)
    assert r is None


def test_get_comments(te):
    r = te.get_comments()
    assert r
    assert r[0]['content'] == '欢迎加入 Typecho 大家族'



from xmlrpc.client import ServerProxy, Fault
from typing import List, Dict, Optional
from dataclasses import asdict

from .log import logger
from .models import Post, Page, Category, Attachment, Comment


class TypechoPostMixin:
    def get_posts(self, num: int = 10) -> Optional[List[Dict]]:
        return self.try_rpc(self.s.metaWeblog.getRecentPosts, num)

    def get_post(self, post_id: int) -> Optional[Dict]:
        return self._try_rpc(self.s.metaWeblog.getPost, post_id, self.username, self.password)

    def new_post(self, post: Post, publish: bool) -> Optional[str]:
        """
        Post's status will cover publish, and if you only save post, the post id will only be '0'
        If Post's categories are not created, it will only create the first category
        """
        return self.try_rpc(self.s.metaWeblog.newPost, post, publish)

    def edit_post(self, post: Post, post_id: int, publish: bool) -> Optional[str]:
        d = asdict(post)
        d.update({'postId': post_id})
        return self.try_rpc(self.s.metaWeblog.newPost, d, publish)

    def del_post(self, post_id: int) -> None:
        return self._try_rpc(self.s.blogger.deletePost, self.blog_id, post_id, self.username, self.password, True)


class TypechoPageMixin:
    def get_pages(self) -> Optional[List[Dict]]:
        return self.try_rpc(self.s.wp.getPages)

    def get_page(self, page_id: int) -> Optional[Dict]:
        """
        WARNING: Different from other API!
        """
        return self._try_rpc(self.s.wp.getPage, self.blog_id, page_id, self.username, self.password)

    def new_page(self, page: Page, publish: bool) -> Optional[str]:
        """
        Page's status will cover publish, and if you only save post, the post id will only be '0'
        """
        return self.try_rpc(self.s.metaWeblog.newPost, page, publish)

    def edit_page(self, page: Page, page_id: int, publish: bool) -> Optional[str]:
        d = asdict(page)
        d.update({'postId': page_id})
        return self.try_rpc(self.s.metaWeblog.newPost, d, publish)

    def del_page(self, page_id: int) -> Optional[bool]:
        return self.try_rpc(self.s.wp.deletePage, page_id)


class TypechoCategoryMixin:
    def get_categories(self) -> Optional[Dict]:
        return self.try_rpc(self.s.metaWeblog.getCategories)

    def new_category(self, category: Category, parent_id: int = 0) -> Optional[str]:
        return self.try_rpc(self.s.wp.newCategory, category)

    def del_category(self, category_id: int) -> Optional[bool]:
        return self.try_rpc(self.s.wp.deleteCategory, category_id)


class TypechoTagMixin:
    def get_tags(self) -> Optional[List[Dict]]:
        return self.try_rpc(self.s.wp.getTags)


class TypechoAttachmentMixin:
    def get_attachments(self, post_id: int = None, mime_type: str = None, page_size: int = None,
                        page_num: int = None) -> Optional[List[Dict]]:
        struct = {}
        if post_id:
            struct.update({'parent_id': post_id})
        if mime_type:
            struct.update({'mime_type': mime_type})
        if page_size:
            struct.update({'number': page_size})
        if page_num:
            struct.update({'offset': page_num})
        return self.try_rpc(self.s.wp.getMediaLibrary, struct)

    def get_attachment(self, attachment_id) -> Optional[Dict]:
        return self.try_rpc(self.s.wp.getMediaItem, attachment_id)

    def new_attachment(self, data: Attachment):
        return self.try_rpc(self.s.wp.uploadFile, data)


class TypechoCommentMixin:
    def get_comments(self, status: str = None, post_id: int = None, page_size: int = None,
                     page_num: int = None) -> Optional[List[Dict]]:
        struct = {}
        if status:
            struct.update({'status': status})
        if post_id:
            struct.update({'parent_id': post_id})
        if page_size:
            struct.update({'number': page_size})
        if page_num:
            struct.update({'offset': page_num})
        return self.try_rpc(self.s.wp.getComments, struct)

    def get_comment(self, comment_id: int) -> Optional[Dict]:
        return self.try_rpc(self.s.wp.getComment, comment_id)

    def new_comment(self, comment: Comment, post_id: int, comment_parent: str = None) -> None:
        d = asdict(comment)
        if comment_parent:
            d.update({'comment_parent': comment_parent})
        path = post_id
        return self.try_rpc(self.s.wp.newComment, path, d)

    def edit_comment(self, comment: Comment, comment_id: int) -> Optional[bool]:
        return self.try_rpc(self.s.wp.editComment, comment_id, comment)

    def del_comment(self, comment_id: int) -> Optional[bool]:
        return self.try_rpc(self.s.wp.deleteComment, comment_id, )


class Typecho(TypechoPostMixin, TypechoPageMixin, TypechoCategoryMixin, TypechoTagMixin, TypechoAttachmentMixin,
              TypechoCommentMixin):
    def __init__(self, rpc_url: str, username: str, password: str):
        self.rpc_url = rpc_url
        self.username = username
        self.password = password

        self.s = ServerProxy(rpc_url)
        # blog id could be any number.
        self.blog_id = 1

    def try_rpc(self, rpc_method, *args, **kw):
        return self._try_rpc(rpc_method, self.blog_id, self.username, self.password, *args, **kw)

    def _try_rpc(self, rpc_method, *args, **kw):
        res = None
        try:
            res = rpc_method(*args, **kw)
            logger.info(res)
            if res == '':
                res = None
        except Fault as e:
            logger.error("Error {}: {}".format(e.faultCode, e.faultString))
        except Exception as e:
            logger.error("Error {}".format(e))
        finally:
            return res
